import { HIGHLIGHTED_COLOR, formatTimestamp } from '@/analyst/utilities'
import DeleteAlert from '@/exercisepanel/DeleteAlert'
import { Button, ButtonGroup, Card } from '@blueprintjs/core'
import { css } from '@emotion/css'
import type { Exercise } from '@inject/graphql/fragments/Exercise.generated'
import { useMemo, useState, type FC, type ReactNode } from 'react'
import LinkButton from '../LinkButton'
import DownloadLogsButton from './DownloadLogsButton'

const card = css`
  display: flex;
  flex-direction: column;
  justify-content: space-between;
  overflow: auto;
`

const infoWrapper = css`
  overflow: auto;
`

const title = css`
  overflow: hidden;
  text-overflow: ellipsis;
`

export type SelectingDetails = {
  onSelect: () => void
}

export type ManagingDetails = {
  runningAny: boolean
  onStart: () => void
  startLoading?: boolean
  onDelete: () => void
  deleteLoading?: boolean
  onPause: () => void
  pauseLoading?: boolean
}

type ExerciseCardProps = {
  isSelected?: boolean
  exercise: Exercise
} & (
  | {
      type: 'selecting'
      details: SelectingDetails
    }
  | { type: 'managing'; details: ManagingDetails }
)

const ExerciseCard: FC<ExerciseCardProps> = ({
  exercise,
  isSelected,
  type,
  details,
}) => {
  const [alertOpen, setAlertOpen] = useState(false)

  const buttons: ReactNode = useMemo(
    () =>
      type === 'selecting' ? (
        !isSelected && (
          <Button onClick={details.onSelect} icon='select'>
            Select
          </Button>
        )
      ) : (
        <>
          {!exercise.running && !exercise.finished && (
            <Button
              icon='play'
              onClick={details.onStart}
              loading={details.startLoading}
              disabled={details.runningAny}
              title={
                details.runningAny
                  ? 'Only one exercise can run at a time'
                  : undefined
              }
            >
              {exercise.exerciseStart ? 'Resume' : 'Start'}
            </Button>
          )}
          {exercise.running && (
            <Button
              icon='pause'
              onClick={details.onPause}
              loading={details.pauseLoading}
            >
              Pause
            </Button>
          )}
          <Button
            icon='trash'
            onClick={() => setAlertOpen(true)}
            disabled={exercise.running}
            title={
              exercise.running ? 'Cannot delete a running exercise' : undefined
            }
          >
            Delete
          </Button>
          <LinkButton
            link={[
              '/exercise-panel/exercise/:exerciseId',
              { params: { exerciseId: exercise.id } },
            ]}
            button={{ icon: 'people', text: 'Participants' }}
          />
          <DownloadLogsButton exerciseId={exercise.id} />
        </>
      ),
    [
      details,
      exercise.exerciseStart,
      exercise.finished,
      exercise.id,
      exercise.running,
      isSelected,
      type,
    ]
  )

  return (
    <Card
      className={card}
      style={{
        backgroundColor: isSelected ? HIGHLIGHTED_COLOR : undefined,
      }}
    >
      <div className={infoWrapper}>
        <h3
          className={title}
          title={exercise.name}
        >{`Exercise: ${exercise.name}`}</h3>
        <h4
          className={title}
          title={
            exercise.definition?.name || exercise.definition?.id?.toString()
          }
        >{`Definition: ${exercise.definition?.name || exercise.definition?.id}`}</h4>
        <p>
          {`${exercise.running ? 'Running' : 'Not running'}, \
                        ${exercise.finished ? 'Finished' : 'Not finished'}`}
        </p>
        <span style={{ fontWeight: 'bold' }}>Team count: </span>
        <span>{exercise.teams.length}</span>
        <br />
        <span style={{ fontWeight: 'bold' }}>Start: </span>
        <span>
          {exercise.exerciseStart
            ? formatTimestamp(exercise.exerciseStart)
            : 'not started yet'}
        </span>
      </div>

      <div style={{ marginTop: '1rem', alignSelf: 'end' }}>
        <ButtonGroup vertical alignText='left'>
          {buttons}
        </ButtonGroup>
      </div>

      {type === 'managing' && (
        <DeleteAlert
          open={alertOpen}
          setOpen={setAlertOpen}
          onDelete={details.onDelete}
          loading={details.deleteLoading}
        >
          <p>
            Are you sure you want to delete the selected exercise? This action
            is irreversible.
          </p>
        </DeleteAlert>
      )}
    </Card>
  )
}
export default ExerciseCard
