import InjectMessage from '@/actionlog/InjectMessage'
import ExitButton from '@/components/ExitButton'
import type { Section } from '@/components/Sidebar'
import Sidebar from '@/components/Sidebar'
import useHideButton from '@/components/Sidebar/useHideButton'
import Status from '@/components/Status'
import { NonIdealState } from '@blueprintjs/core'
import Manager from '@inject/graphql/components/Manager'
import type { ActionLog } from '@inject/graphql/fragments/ActionLog.generated'
import { useGetExerciseChannels } from '@inject/graphql/queries/GetExerciseChannels.generated'
import { useGetExerciseConfig } from '@inject/graphql/queries/GetExerciseConfig.generated'
import { useGetTeam } from '@inject/graphql/queries/GetTeam.generated'
import useEmailsEnabled from '@inject/graphql/utils/useEmailsEnabled'
import useExerciseSubscriptionStatus from '@inject/graphql/utils/useExerciseSubscriptionStatus'
import NotificationDropdown from '@inject/shared/notification/NotificationDropdown'
import notEmpty from '@inject/shared/utils/notEmpty'
import type { FC, PropsWithChildren } from 'react'
import { useCallback } from 'react'
import { useLocation } from 'react-router-dom'
import TraineeEmailFormOverlay from '../../email/EmailFormOverlay/TraineeEmailFormOverlay'
import ChannelButton from '../ChannelButton'

interface TraineeViewProps extends PropsWithChildren {
  teamId: string
  exerciseId: string
  selectedThreadId?: string
}

const TraineeView: FC<TraineeViewProps> = ({
  children,
  teamId,
  exerciseId,
  selectedThreadId,
}) => {
  const { hide: hideLeftBar, node: hideButton } = useHideButton()
  const loc = useLocation()
  const { data } = useGetTeam({ variables: { teamId } })

  const { data: dataExerciseConfig } = useGetExerciseConfig({
    variables: {
      exerciseId,
      teamId,
    },
  })

  const emailsEnabled = useEmailsEnabled({
    variables: {
      exerciseId,
    },
  })

  const { data: channelsData } = useGetExerciseChannels({
    variables: { exerciseId },
  })
  const channels = channelsData?.exerciseChannels?.filter(notEmpty) || []
  const exerciseLoopRunning = useExerciseSubscriptionStatus(exerciseId || '')

  const getPopup = useCallback(
    (actionLog: ActionLog) => (
      <InjectMessage
        actionLog={actionLog}
        exerciseId={exerciseId}
        teamId={teamId}
        inInstructor={false}
        allowFileRedirect={false}
        fill
      />
    ),
    [exerciseId, teamId]
  )

  const sections: Section[] = [
    {
      node: (
        <Status
          showTime={
            dataExerciseConfig?.exerciseConfig?.showExerciseTime || false
          }
          small={hideLeftBar}
          exerciseRunning={exerciseLoopRunning || false}
          team={data?.team || undefined}
          hideLabel={hideLeftBar}
        />
      ),
    },

    {
      name: 'Options',
      node: (
        <>
          {hideButton}
          <NotificationDropdown hideLabel={hideLeftBar} fill />
          <ExitButton hideLabel={hideLeftBar} />
        </>
      ),
    },
    {
      name: 'Channels',
      node: (
        <>
          {exerciseLoopRunning &&
            channels.map(channel => (
              <ChannelButton
                key={channel.id}
                channel={channel}
                exerciseId={exerciseId}
                linkTo='trainee'
                pathname={loc.pathname}
                teamId={teamId}
                hideLabel={hideLeftBar}
              />
            ))}
        </>
      ),
    },
  ]

  return (
    <>
      {emailsEnabled && (
        <TraineeEmailFormOverlay teamId={teamId} exerciseId={exerciseId} />
      )}
      <Manager getPopup={getPopup} selectedThreadId={selectedThreadId} />

      <div style={{ height: '100%', display: 'flex' }}>
        <Sidebar
          position='left'
          sections={sections}
          hideNames={hideLeftBar}
          showLogo
        />

        <div style={{ overflow: 'auto', flex: 1 }}>
          {exerciseLoopRunning ? (
            children
          ) : (
            <NonIdealState
              icon='disable'
              title='Exercise not running'
              description='The exercise is not running, please contact your instructor.'
            />
          )}
        </div>
      </div>
    </>
  )
}

export default TraineeView
